#include <avr/io.h>
#include <util/delay.h>

#define RELAY_ON PORTB|=(1<<PORTB4) //zaczenie przekanika
#define RELAY_OFF PORTB&=~(1<<PORTB4) //wyczenie przekanika

//sterowanie diod nadawcz
#define IR_OFF PORTB&=~(1<<PORTB0);TCCR0A=0;TCCR0B=0
#define IR_ON TCCR0A=(1<<COM0A0)|(1<<WGM01);TCCR0B=(1<<CS00)

#define MODE_REFLECT bit_is_set(PINB,PINB2) //tryb pracy: odbiciowy
#define MODE_TRANSMIT bit_is_clear(PINB,PINB2) //tryb pracy: transmisyjny

#define PULSE_LO bit_is_clear(PINB,PINB3) //stan wyjcia odbiornika: czy niski?

#define DELAY_LOOP 400 //czas nadawania w mikrosekundach
#define TIMES_LOOP 5 //liczba nadanych "paczek"
#define DECISION 2 //ile powtrze ptli jest potrzebne do podjcia decyzji
#define DELAY_AFTER_CHANGE_STATE 500 //opnienie w milisekundach po przeczeniu
#define DELAY_MAIN 5 //opnienie ptli gwnej w milisekundach
#define OCR_VALUE 121 //dla uzyskania 36kHz


volatile uint8_t loop; //licznik wysanych "paczek"
volatile uint8_t counter; //licznik poprawnie odebranych "paczek"
volatile uint8_t onoff = 0; //0 - przekanik wyczony, 1 zaczony
volatile uint8_t decide_cnt = 0; //licznik do podjcia decyzji o przeczeniu
//0 - stan spoczynku
//1 - zgoda na przeczenie przekanika
//2 - oczekiwanie po przeczeniu przekanika
//3 - oczekiwanie na powrt do stanu spoczynku
volatile uint8_t change_state = 0;

//=================================================
//=================================================
//=================================================

int main(void)
{

	DDRB = (1 << DDB4) | (1 << DDB0); //konfiguracja portw
	PORTB = (1 << PORTB2);
	
	OCR0A = OCR_VALUE; //ustawienie wartoci rejestru OCR0A


	while(1)
	{	
		
		counter = 0; //przygotowanie zmiennych
		loop = TIMES_LOOP;
		while(loop)
		{
			loop--;
			IR_ON; //zaczenie diody nadawczej
			_delay_us(DELAY_LOOP); //oczekiwanie na ustalenie stanu logicznego wyjcia odbiornika
			if(PULSE_LO) //jeeli reakcja jest prawidowa
			{
				counter++; //inkrementuj licznik poprawnie odebranych "paczek"
			}
			IR_OFF; //wycz diod nadawcz
			_delay_us(5*DELAY_LOOP); //oczekiwanie midzy poszczeglnymi "paczkami"
		}
		
		if(MODE_REFLECT) //w trybie odbiciowym
		{
			if(counter == TIMES_LOOP) //jeeli odebrano prawidow paczk impulsw
			{
				if(decide_cnt < DECISION) //zainkrementuj - z ograniczeniem do nasycenia
				{
					decide_cnt++;
				}
			}
			else //jeeli paczka jest nieprawidowa
			{
				decide_cnt = 0; //wr do stanu spoczynku
				change_state = 0;
			}
		}
		
		if(MODE_TRANSMIT) //w trybie transmisyjnym
		{
			if(counter < TIMES_LOOP) //jeeli odebrano nieprawidow paczk impulsw
			{
				if(decide_cnt < DECISION) //zainkrementuj - z ograniczeniem do nasycenia
				{
					decide_cnt++;
				}
			}
			else //jeeli paczka jest prawidowa
			{
				decide_cnt = 0; //wr do stanu spoczynku
				change_state = 0;
			}
		}
		
		
		if(decide_cnt == DECISION && change_state == 0) //po podjciu decyzji o zmianie stanu
		{
			change_state = 1; //wyra zgod na zmian stanu przekanika
		}	
			
		if(change_state == 1)
		{
			onoff ^= 1; //przeczenie przekanika w stan przeciwny
			change_state = 2;
		}
		
		
		if(onoff)
		{
			RELAY_ON;
		}
		else
		{
			RELAY_OFF;
		}
		
		if(change_state == 2) //oczekiwanie po przeczeniu, aby reakcja ukadu bya natychmiastowa
		{
			_delay_ms(DELAY_AFTER_CHANGE_STATE);
			change_state = 3;
		}
		else
		{
			_delay_ms(DELAY_MAIN); //w stanie spoczynku ustal odstpy midzy grupami "paczek"
		}
		
	}
}